/* Pango
 * pangogp-private.h:
 *
 * Copyright (C) 1999 Red Hat Software
 * Copyright (C) 2000 Tor Lillqvist
 * Copyright (C) 2001 Sun Microsystems
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __PANGOGP_PRIVATE_H__
#define __PANGOGP_PRIVATE_H__

#include "pango-modules.h"
#include "pangogp.h"
#include <fontconfig/fontconfig.h>

/* Debugging... */
/*#define DEBUGGING 1*/

#if defined(DEBUGGING) && DEBUGGING
#ifdef __GNUC__
#define PING(printlist)					\
(g_print ("%s:%d ", __PRETTY_FUNCTION__, __LINE__),	\
 g_print printlist,					\
 g_print ("\n"))
#else
#define PING(printlist)					\
(g_print ("%s:%d ", __FILE__, __LINE__),		\
 g_print printlist,					\
 g_print ("\n"))
#endif
#else  /* !DEBUGGING */
#define PING(printlist)
#endif

#define PANGO_RESOLUTION 300
#define PANGO_SCALE_26_6 (PANGO_SCALE / (1<<6))
#define PANGO_PIXELS_26_6(d)				\
  (((d) >= 0) ?						\
   ((d) + PANGO_SCALE_26_6 / 2) / PANGO_SCALE_26_6 :	\
   ((d) - PANGO_SCALE_26_6 / 2) / PANGO_SCALE_26_6)
#define PANGO_UNITS_26_6(d) (PANGO_SCALE_26_6 * (d))
#define PANGO_UNITS_26_6_PX(d) (PANGO_SCALE_26_6 * 72 * (d) / PANGO_RESOLUTION)

typedef struct _PangoGPBaselineTable PangoGPBaselineTable;
typedef struct _PangoGPAttrBaseline  PangoGPAttrBaseline;
typedef struct _PangoGPFont          PangoGPFont;
typedef struct _PangoGPGlyphInfo     PangoGPGlyphInfo;

struct _PangoGPAttrBaseline
{
  PangoAttribute attr;
  PangoGPBaseline baseline;
  PangoFontDescription *desc;
};

typedef struct _PangoGPAttrPointer PangoGPAttrPointer;

struct _PangoGPAttrPointer
{
  PangoAttribute attr;
  gpointer pointer;
};

struct _PangoGPBaselineTable
{
  gint units_per_EM;

  gint alphabetic;
  gint ideographic;
  gint hanging;
  gint mathematical;
  gint central;
  gint middle;
  gint text_before_edge;
  gint text_after_edge;
};

struct _PangoGPFont
{
  PangoFont font;

  FcPattern *font_pattern;
  FT_Face face;
  int load_flags;

  GnomeFontFace *gnome_face;
  GnomeFont     *gnome_font;
  gchar *postscript_name;
  gboolean embed;

  double size;

  int font_handle;
  int agl1_font_handle;
  int agl2_font_handle;
  int agl3_font_handle;
  int agl4_font_handle;

  PangoFontMap *fontmap;
  PangoFontDescription *description;

  GSList *metrics_by_lang;

  /* If TRUE, font is in cache of recently unused fonts and not otherwise
   * in use. */
  gboolean in_cache;

  GHashTable *glyph_info;
  GDestroyNotify glyph_cache_destroy;
};

struct _PangoGPGlyphInfo
{
  PangoRectangle logical_rect;
  PangoRectangle ink_rect;
  void *cached_glyph;
};

PangoGPFont * _pango_gp_font_new                (PangoFontMap                *font,
						   FcPattern              *pattern);
PangoMap      *_pango_gp_get_shaper_map          (PangoLanguage     *language);
PangoCoverage *_pango_gp_font_map_get_coverage   (PangoFontMap                *fontmap,
						   FcPattern                   *pattern);
FT_Library     _pango_gp_font_map_get_library    (PangoFontMap      *fontmap);
void           _pango_gp_font_map_cache_add      (PangoFontMap      *fontmap,
						   PangoGPFont      *gpfont);
void           _pango_gp_font_map_cache_remove    (PangoFontMap      *fontmap,
						   PangoGPFont      *gpfont);
void           _pango_gp_font_map_add            (PangoFontMap      *fontmap,
						   PangoGPFont      *gpfont);
void           _pango_gp_font_map_remove         (PangoFontMap      *fontmap,
						   PangoGPFont      *gpfont);
const char    *_pango_gp_ft_strerror             (FT_Error           error);
PangoFontDescription *_pango_gp_font_desc_from_pattern (FcPattern *pattern,
							 gboolean        include_size);

void *pango_gp_font_get_cache_glyph_data    (PangoFont      *font,
					      int             glyph_index);
void  pango_gp_font_set_cache_glyph_data    (PangoFont      *font,
					      int             glyph_index,
					      void           *cached_glyph);
void  pango_gp_font_set_glyph_cache_destroy (PangoFont      *font,
					      GDestroyNotify  destroy_notify);

PangoGPBaselineTable* pango_gp_font_baseline_table_get (PangoGPFont *font);

PangoLayout *pango_gp_layout_new (PangoContext *context);

#endif /* __PANGOGP_PRIVATE_H__ */
