/* Pango
 * pangogp.h: Routines for handling FreeType2 fonts with Gp library
 *
 * Copyright (C) 1999 Red Hat Software
 * Copyright (C) 2000 Tor Lillqvist
 * Copyright (C) 2001 Sun Microsystems
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __PANGOGP_H__
#define __PANGOGP_H__

#include <libgnomeprint/gnome-print.h>

#include <ft2build.h>
#include FT_FREETYPE_H

#include <fontconfig/fontconfig.h>

#include <pango/pango-layout.h>
#include <pango/pango-attributes.h>

G_BEGIN_DECLS

typedef enum {
  PANGO_GP_BASELINE_INVALID,
  PANGO_GP_BASELINE_AUTO,
  PANGO_GP_BASELINE_RESET_SIZE,
  PANGO_GP_BASELINE_ALPHABETIC,
  PANGO_GP_BASELINE_IDEOGRAPHIC,
  PANGO_GP_BASELINE_HANGING,
  PANGO_GP_BASELINE_MATHEMATICAL,
  PANGO_GP_BASELINE_CENTRAL,
  PANGO_GP_BASELINE_MIDDLE,
  PANGO_GP_BASELINE_TEXT_BEFORE_EDGE,
  PANGO_GP_BASELINE_TEXT_AFTER_EDGE,
  PANGO_GP_BASELINE_BEFORE_EDGE,
  PANGO_GP_BASELINE_AFTER_EDGE,
  PANGO_GP_BASELINE_LENGTH,
  PANGO_GP_BASELINE_USE_SCRIPT,
  PANGO_GP_BASELINE_BASELINE
} PangoGPBaseline;

typedef enum {
  PANGO_GP_LINE_STACKING_STRATEGY_INVALID,
  PANGO_GP_LINE_STACKING_STRATEGY_LINE_HEIGHT,
  PANGO_GP_LINE_STACKING_STRATEGY_FONT_HEIGHT,
  PANGO_GP_LINE_STACKING_STRATEGY_MAX_HEIGHT
} PangoGPLineStackingStrategy;

typedef enum {
  PANGO_GP_FONT_EMBED_INVALID,
  PANGO_GP_FONT_EMBED_ALL,
  PANGO_GP_FONT_EMBED_NONBASE,
  PANGO_GP_FONT_EMBED_NONE
} PangoGPFontEmbed;

#define PANGO_RENDER_TYPE_GP "PangoRenderGP"

#define PANGO_TYPE_GP_FONT_MAP              (pango_gp_font_map_get_type ())
#define PANGO_GP_FONT_MAP(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_GP_FONT_MAP, PangoGPFontMap))
#define PANGO_GP_IS_FONT_MAP(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_GP_FONT_MAP))

typedef struct _PangoGPFontMap      PangoGPFontMap;
typedef struct _PangoGPLayout      PangoGPLayout;
typedef struct _PangoGPLayoutClass PangoGPLayoutClass;

typedef void (*PangoGPSubstituteFunc) (FcPattern *pattern,
				       gpointer   data);

/* Calls for applications */
PangoContext      *pango_gp_get_context    (GnomePrintContext *gpc);
PangoFontMap      *pango_gp_font_map_for_display (void);
void               pango_gp_shutdown_display     (void);


void           pango_gp_render             (PangoContext *context,
					    const gchar  *layout_text,
					    PangoLayoutRun   *run,
					    PangoFont           *font,
					    PangoGlyphString    *glyphs,
					    gint               x,
					    gint               y);
void           pango_gp_render_layout_line (PangoContext *context,
					    const gchar   *layout_text,
					    PangoLayoutLine  *line,
					    gint              x,
					    gint         y);
void 	       pango_gp_render_layout_lines (PangoContext *context,
					     PangoLayout *layout,
					     gint	      line_first,
					     gint         line_last,
					     gint         x, 
					     gint         y);
void           pango_gp_render_layout       (PangoContext *context,
					     PangoLayout      *layout,
					     gint              x, 
					     gint              y);

GType pango_gp_font_map_get_type (void);

PangoFontMap *pango_gp_font_map_new                    (void);
void          pango_gp_font_map_set_resolution         (PangoGPFontMap        *fontmap,
							 double                  dpi_x,
							 double                  dpi_y);
PangoGPFontEmbed pango_gp_font_map_get_font_embed (PangoGPFontMap *font_map);
void          pango_gp_font_map_set_font_embed (PangoGPFontMap      *font_map,
						    PangoGPFontEmbed font_embed);
void          pango_gp_font_map_set_default_substitute (PangoGPFontMap        *fontmap,
							    PangoGPSubstituteFunc  func,
							    gpointer                   data,
							    GDestroyNotify             notify);
void          pango_gp_font_map_substitute_changed     (PangoGPFontMap         *fontmap);
PangoContext *pango_gp_font_map_create_context         (PangoGPFontMap         *fontmap);

typedef enum
{
  PANGO_GP_DEBUG_NONE	= 0,
  PANGO_GP_DEBUG_DEBUG	= 1,
  PANGO_GP_DEBUG_MASK	= 0x01
} PangoGPDebugFlags;

PangoContext *pango_gp_font_map_create_context_with_debug_flags (PangoGPFontMap *fontmap,
								     PangoGPDebugFlags debug_flags);


PangoAttrType pango_gp_attr_alignment_adjust_get_type (void);
PangoAttrType pango_gp_attr_alignment_baseline_get_type (void);
PangoAttrType pango_gp_attr_db_reset_size_get_type (void);
PangoAttrType pango_gp_attr_dominant_baseline_get_type (void);
PangoAttrType pango_gp_attr_callback_get_type (void);
PangoAttrType pango_gp_attr_keep_together_within_line_get_type (void);
PangoAttrType pango_gp_attr_keep_with_next_within_line_get_type (void);
PangoAttrType pango_gp_attr_keep_with_previous_within_line_get_type (void);
PangoAttrType pango_gp_attr_line_height_get_type (void);
PangoAttribute *pango_gp_attr_alignment_adjust_new (gint alignment_adjust);
PangoAttribute *pango_gp_attr_db_reset_size_new (gboolean db_reset_size);
PangoAttribute *pango_gp_attr_alignment_baseline_new (gint alignment_baseline);
PangoAttribute *pango_gp_attr_dominant_baseline_new (PangoGPBaseline dominant_baseline,
							 PangoFontDescription *font_desc);
PangoAttribute *pango_gp_attr_callback_new (gpointer callback);
gpointer        pango_gp_attr_callback_get_callback (const PangoAttribute *attr);
PangoAttribute *pango_gp_attr_keep_together_within_line_new (gint strength);
PangoAttribute *pango_gp_attr_keep_with_next_within_line_new (gint strength);
PangoAttribute *pango_gp_attr_keep_with_previous_within_line_new (gint strength);
PangoAttribute *pango_gp_attr_line_height_new (gint line_height);
PangoAttribute *pango_gp_attr_line_stacking_strategy_new (PangoGPLineStackingStrategy line_stacking_strategy);

void pango_gp_layout_set_line_height (PangoLayout *layout, gint line_height);
void pango_gp_layout_set_line_stacking_strategy (PangoLayout *layout,
						     PangoGPLineStackingStrategy line_stacking_strategy);

gint pango_gp_layout_get_item_attrs (PangoItem *item,
					 PangoAttrType first_attribute_type,
					 ...);

/* API for rendering modules
 */
PangoGlyph     pango_gp_get_unknown_glyph (PangoFont       *font);
int            pango_gp_font_get_kerning  (PangoFont       *font,
					    PangoGlyph       left,
					    PangoGlyph       right);
FT_Face        pango_gp_font_get_face     (PangoFont       *font);
PangoCoverage *pango_gp_font_get_coverage (PangoFont       *font,
					    PangoLanguage   *language);

G_END_DECLS

#endif /* __PANGOGP_H__ */
