/* Pango
 * pango-context.c: Contexts for itemization and shaping
 *
 * Copyright (C) 2000 Red Hat Software
 * Copyright (C) 2003 Sun Microsystems
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "pango/pango-context-private.h"
#include "pango/pangopdflib-context.h"
#include "pango/pangopdflib-private.h"

struct _PangoPDFLIBContext
{
  PangoContext             parent_instance;

  PangoPDFLIBDebugFlags    debug_flags;
};

struct _PangoPDFLIBContextClass
{
  PangoContextClass        parent_class;
};

static void pango_pdflib_context_class_init  (PangoPDFLIBContextClass *klass);

static gpointer parent_class;

GType
pango_pdflib_context_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (PangoPDFLIBContextClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) pango_pdflib_context_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (PangoPDFLIBContext),
        0,              /* n_preallocs */
	NULL,
	NULL
      };
      
      object_type = g_type_register_static (PANGO_TYPE_CONTEXT,
                                            "PangoPDFLIBContext",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
pango_pdflib_context_class_init (PangoPDFLIBContextClass *klass)
{
  parent_class = g_type_class_peek_parent (klass);

  PANGO_CONTEXT_CLASS (klass)->layout_new = pango_pdflib_layout_new;
}

/**
 * pango_pdflib_context_new:
 * 
 * Creates a new #PangoPDFLIBContext initialized to default value.
 * 
 * Return value: the new #PangoPDFLIBContext.
 **/
PangoContext *
pango_pdflib_context_new (void)
{
  PangoContext *context;

  context =
    PANGO_CONTEXT (g_object_new (pango_pdflib_context_get_type (),
				 NULL));
  
  return context;
}

PangoPDFLIBDebugFlags
pango_pdflib_context_get_debug_flags (PangoContext *context)
{
  g_return_val_if_fail (PANGO_IS_PDFLIB_CONTEXT (context),
			PANGO_PDFLIB_DEBUG_NONE);

  return (PANGO_PDFLIB_CONTEXT (context)->debug_flags);
}

void
pango_pdflib_context_set_debug_flags (PangoContext         *context,
				      PangoPDFLIBDebugFlags debug_flags)
{
  g_return_if_fail (PANGO_IS_PDFLIB_CONTEXT (context));

  PANGO_PDFLIB_CONTEXT (context)->debug_flags = debug_flags;
}
